/*****************************************************************************
 *   timer.c: Timer C file for NXP LPC288x Family Microprocessors
 *
 *   Copyright(C) 2007, NXP Semiconductor
 *   All rights reserved.
 *
 *   History
 *   2007.02.01  ver 1.00    Prelimnary version, first Release
 *
******************************************************************************/
#include "LPC288x.h"			/* LPC318X Peripheral Registers	*/
#include "type.h"
#include "target.h"
#include "irq.h"
#include "timer.h"

volatile DWORD timer_counter = 0;

/******************************************************************************
** Function name:		Timer0Handler
**
** Descriptions:		Timer/Counter 0 interrupt handler
**
**
** parameters:			None
** Returned value:		None
** 
******************************************************************************/
void IRQ_Timer( void )
{
  timer_counter++;		
  /* Clear the timer interrupt */
  T0CLR=0x0;
  return;
}

/******************************************************************************
** Function name:		init_timer
**
** Descriptions:		Timer/Counter 0 initialization
**
**
** parameters:			None
** Returned value:		TRUE or FALSE
** 
******************************************************************************/
DWORD init_timer( DWORD TimeInterval )
{  		
  /* Configure Interrupt Controller 	  */
  if ( install_IRQ(5, 1, IRQ_Timer ) == FALSE )
  {
	return( FALSE );
  }
  INT_REQ5 = (1<<28)|(1<<27)|(1<<26)|(1<<16)|0x1;
  INT_VECTOR0 = IRQ_TABLE_BASE & MASK_INDEX;

  /* Configure Timer0 */
  T0LOAD = TimeInterval;	/* Timer value  */
  T0VALUE = TimeInterval;
  T0CLR = 0;
  T0CTRL=0xC8;
  return( TRUE );
}

/******************************************************************************
**                            End Of File
******************************************************************************/
